Class: BoltServer::TransportApp

Inherits:
Sinatra::Base
  • Object
show all
Defined in:
lib/bolt_server/transport_app.rb

Constant Summary collapse

PARTIAL_SCHEMAS =

These partial schemas are reused to build multiple request schemas

%w[target-any target-ssh target-winrm task].freeze
REQUEST_SCHEMAS =

These schemas combine shared schemas to describe client requests

%w[
  action-check_node_connections
  action-run_command
  action-run_task
  action-run_script
  action-upload_file
  transport-ssh
  transport-winrm
].freeze
ACTIONS =
%w[
  check_node_connections
  run_command
  run_task
  run_script
  upload_file
].freeze

Instance Method Summary collapse

Constructor Details

#initialize(config) ⇒ TransportApp



37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
# File 'lib/bolt_server/transport_app.rb', line 37

def initialize(config)
  @config = config
  @schemas = Hash[REQUEST_SCHEMAS.map do |basename|
    [basename, JSON.parse(File.read(File.join(__dir__, ['schemas', "#{basename}.json"])))]
  end]

  PARTIAL_SCHEMAS.each do |basename|
    schema_content = JSON.parse(File.read(File.join(__dir__, ['schemas', 'partials', "#{basename}.json"])))
    shared_schema = JSON::Schema.new(schema_content, Addressable::URI.parse("partial:#{basename}"))
    JSON::Validator.add_schema(shared_schema)
  end

  @executor = Bolt::Executor.new(0)

  @file_cache = BoltServer::FileCache.new(@config).setup

  # This is needed until the PAL is threadsafe.
  @pal_mutex = Mutex.new

  super(nil)
end

Instance Method Details

#build_puppetserver_uri(file_identifier, module_name, environment) ⇒ Object



224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
# File 'lib/bolt_server/transport_app.rb', line 224

def build_puppetserver_uri(file_identifier, module_name, environment)
  segments = file_identifier.split('/', 3)
  if segments.size == 1
    {
      'path' => "/puppet/v3/file_content/tasks/#{module_name}/#{file_identifier}",
      'params' => {
        'environment' => environment
      }
    }
  else
    module_segment, mount_segment, name_segment = *segments
    {
      'path' => case mount_segment
                when 'files'
                  "/puppet/v3/file_content/modules/#{module_segment}/#{name_segment}"
                when 'tasks'
                  "/puppet/v3/file_content/tasks/#{module_segment}/#{name_segment}"
                when 'lib'
                  "/puppet/v3/file_content/plugins/#{name_segment}"
                end,
      'params' => {
        'environment' => environment
      }
    }
  end
end

#check_node_connections(targets, body) ⇒ Object



126
127
128
129
130
131
132
133
134
135
136
137
# File 'lib/bolt_server/transport_app.rb', line 126

def check_node_connections(targets, body)
  error = validate_schema(@schemas["action-check_node_connections"], body)
  return [], error unless error.nil?

  # Puppet Enterprise's orchestrator service uses the
  # check_node_connections endpoint to check whether nodes that should be
  # contacted over SSH or WinRM are responsive. The wait time here is 0
  # because the endpoint is meant to be used for a single check of all
  # nodes; External implementations of wait_until_available (like
  # orchestrator's) should contact the endpoint in their own loop.
  [@executor.wait_until_available(targets, wait_time: 0), nil]
end

#error_result(error) ⇒ Object



66
67
68
69
70
71
# File 'lib/bolt_server/transport_app.rb', line 66

def error_result(error)
  {
    'status' => 'failure',
    'value' => { '_error' => error.to_h }
  }
end

#in_pe_pal_env(environment) ⇒ Object



193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
# File 'lib/bolt_server/transport_app.rb', line 193

def in_pe_pal_env(environment)
  if environment.nil?
    [400, '`environment` is a required argument']
  else
    @pal_mutex.synchronize do
      pal = BoltServer::PE::PAL.new({}, environment)
      yield pal
    rescue Puppet::Environments::EnvironmentNotFound
      [400, {
        "class" => 'bolt/unknown-environment',
        "message" => "Environment #{environment} not found"
      }.to_json]
    rescue Bolt::Error => e
      [400, e.to_json]
    end
  end
end

#make_ssh_target(target_hash) ⇒ Object



306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
# File 'lib/bolt_server/transport_app.rb', line 306

def make_ssh_target(target_hash)
  defaults = {
    'host-key-check' => false
  }

  overrides = {
    'load-config' => false
  }

  opts = defaults.merge(target_hash).merge(overrides)

  if opts['private-key-content']
    private_key_content = opts.delete('private-key-content')
    opts['private-key'] = { 'key-data' => private_key_content }
  end

  data = {
    'uri' => target_hash['hostname'],
    'config' => {
      'transport' => 'ssh',
      'ssh' => opts
    }
  }

  inventory = Bolt::Inventory.empty
  Bolt::Target.from_hash(data, inventory)
end

#make_winrm_target(target_hash) ⇒ Object



354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
# File 'lib/bolt_server/transport_app.rb', line 354

def make_winrm_target(target_hash)
  defaults = {
    'ssl' => false,
    'ssl-verify' => false
  }

  opts = defaults.merge(target_hash)

  data = {
    'uri' => target_hash['hostname'],
    'config' => {
      'transport' => 'winrm',
      'winrm' => opts
    }
  }

  inventory = Bolt::Inventory.empty
  Bolt::Target.from_hash(data, inventory)
end

#pe_plan_info(pal, module_name, plan_name) ⇒ Object



211
212
213
214
215
216
217
218
219
220
221
222
# File 'lib/bolt_server/transport_app.rb', line 211

def pe_plan_info(pal, module_name, plan_name)
  # Handle case where plan name is simply module name with special `init.pp` plan
  plan_name = if plan_name == 'init' || plan_name.nil?
                module_name
              else
                "#{module_name}::#{plan_name}"
              end
  plan_info = pal.get_plan_info(plan_name)
  # Path to module is meaningless in PE
  plan_info.delete('module')
  plan_info
end

#pe_task_info(pal, module_name, task_name, environment) ⇒ Object



251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
# File 'lib/bolt_server/transport_app.rb', line 251

def pe_task_info(pal, module_name, task_name, environment)
  # Handle case where task name is simply module name with special `init` task
  task_name = if task_name == 'init' || task_name.nil?
                module_name
              else
                "#{module_name}::#{task_name}"
              end
  task = pal.get_task(task_name)
  files = task.files.map do |file_hash|
    {
      'filename' => file_hash['name'],
      'sha256' => Digest::SHA256.hexdigest(File.read(file_hash['path'])),
      'size_bytes' => File.size(file_hash['path']),
      'uri' => build_puppetserver_uri(file_hash['name'], module_name, environment)
    }
  end
  {
    'metadata' => task.,
    'name' => task.name,
    'files' => files
  }
end

#result_set_to_data(result_set, aggregate: false) ⇒ Object

Turns a Bolt::ResultSet object into a status hash that is fit to return to the client in a response.

If the result_set has more than one result, the status hash will have a status value and a list of target results. If the result_set contains only one item, it will be returned as a single result object. Set aggregate to treat it as a set of results with length 1 instead.



90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
# File 'lib/bolt_server/transport_app.rb', line 90

def result_set_to_data(result_set, aggregate: false)
  scrubbed_results = result_set.map do |result|
    scrub_stack_trace(result.to_data)
  end

  if aggregate || scrubbed_results.length > 1
    # For actions that act on multiple targets, construct a status hash for the aggregate result
    all_succeeded = scrubbed_results.all? { |r| r['status'] == 'success' }
    {
      status: all_succeeded ? 'success' : 'failure',
      result: scrubbed_results
    }
  else
    # If there was only one target, return the first result on its own
    scrubbed_results.first
  end
end

#run_command(target, body) ⇒ Object



118
119
120
121
122
123
124
# File 'lib/bolt_server/transport_app.rb', line 118

def run_command(target, body)
  error = validate_schema(@schemas["action-run_command"], body)
  return [], error unless error.nil?

  command = body['command']
  [@executor.run_command(target, command), nil]
end

#run_script(target, body) ⇒ Object



183
184
185
186
187
188
189
190
191
# File 'lib/bolt_server/transport_app.rb', line 183

def run_script(target, body)
  error = validate_schema(@schemas["action-run_script"], body)
  return [], error unless error.nil?

  # Download the file onto the machine.
  file_location = @file_cache.update_file(body['script'])

  [@executor.run_script(target, file_location, body['arguments'])]
end

#run_task(target, body) ⇒ Object



108
109
110
111
112
113
114
115
116
# File 'lib/bolt_server/transport_app.rb', line 108

def run_task(target, body)
  error = validate_schema(@schemas["action-run_task"], body)
  return [], error unless error.nil?

  task_data = body['task']
  task = Bolt::Task::PuppetServer.new(task_data['name'], task_data['metadata'], task_data['files'], @file_cache)
  parameters = body['parameters'] || {}
  [@executor.run_task(target, task, parameters), nil]
end

#scrub_stack_trace(result) ⇒ Object



59
60
61
62
63
64
# File 'lib/bolt_server/transport_app.rb', line 59

def scrub_stack_trace(result)
  if result.dig('value', '_error', 'details', 'stack_trace')
    result['value']['_error']['details'].reject! { |k| k == 'stack_trace' }
  end
  result
end

#upload_file(target, body) ⇒ Object



139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
# File 'lib/bolt_server/transport_app.rb', line 139

def upload_file(target, body)
  error = validate_schema(@schemas["action-upload_file"], body)
  return [], error unless error.nil?

  files = body['files']
  destination = body['destination']
  job_id = body['job_id']
  cache_dir = @file_cache.create_cache_dir(job_id.to_s)
  FileUtils.mkdir_p(cache_dir)
  files.each do |file|
    relative_path = file['relative_path']
    uri = file['uri']
    sha256 = file['sha256']
    kind = file['kind']
    path = File.join(cache_dir, relative_path)
    case kind
    when 'file'
      # The parent should already be created by `directory` entries,
      # but this is to be on the safe side.
      parent = File.dirname(path)
      FileUtils.mkdir_p(parent)
      @file_cache.serial_execute { @file_cache.download_file(path, sha256, uri) }
    when 'directory'
      # Create directory in cache so we can move files in.
      FileUtils.mkdir_p(path)
    else
      return [400, Bolt::Error.new("Invalid `kind` of '#{kind}' supplied. Must be `file` or `directory`.",
                                   'boltserver/schema-error').to_json]
    end
  end
  # We need to special case the scenario where only one file was
  # included in the request to download. Otherwise, the call to upload_file
  # will attempt to upload with a directory as a source and potentially a
  # filename as a destination on the host. In that case the end result will
  # be the file downloaded to a directory with the same name as the source
  # filename, rather than directly to the filename set in the destination.
  upload_source = if files.size == 1 && files[0]['kind'] == 'file'
                    File.join(cache_dir, files[0]['relative_path'])
                  else
                    cache_dir
                  end
  [@executor.upload_file(target, upload_source, destination), nil]
end

#validate_schema(schema, body) ⇒ Object



73
74
75
76
77
78
79
80
# File 'lib/bolt_server/transport_app.rb', line 73

def validate_schema(schema, body)
  schema_error = JSON::Validator.fully_validate(schema, body)
  if schema_error.any?
    Bolt::Error.new("There was an error validating the request body.",
                    'boltserver/schema-error',
                    schema_error)
  end
end