Method: Bolt::Config::Modulepath#user_modulepath
- Defined in:
- lib/bolt/config/modulepath.rb
#user_modulepath ⇒ Object (readonly)
The user_modulepath only includes the original modulepath and is used during pluginsync. We don’t want to pluginsync any of the content from BOLT_MODULES since that content includes core modules that can conflict with modules installed with an agent.
14 15 16 |
# File 'lib/bolt/config/modulepath.rb', line 14 def user_modulepath @user_modulepath end |