Class: Bolt::Target

Inherits:
Object
  • Object
show all
Defined in:
lib/bolt/target.rb

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(uri, options = nil) ⇒ Target

Returns a new instance of Target.



13
14
15
16
17
18
# File 'lib/bolt/target.rb', line 13

def initialize(uri, options = nil)
  @uri = uri
  @uri_obj = parse(uri)
  @options = options || {}
  @options.freeze
end

Instance Attribute Details

#optionsObject (readonly)

Returns the value of attribute options.



6
7
8
# File 'lib/bolt/target.rb', line 6

def options
  @options
end

#uriObject (readonly)

Returns the value of attribute uri.



6
7
8
# File 'lib/bolt/target.rb', line 6

def uri
  @uri
end

Class Method Details

.from_asserted_hash(hash) ⇒ Object

Satisfies the Puppet datatypes API



9
10
11
# File 'lib/bolt/target.rb', line 9

def self.from_asserted_hash(hash)
  new(hash['uri'], hash['options'])
end

Instance Method Details

#eql?(other) ⇒ Boolean Also known as: ==

Returns:

  • (Boolean)


45
46
47
# File 'lib/bolt/target.rb', line 45

def eql?(other)
  self.class.equal?(other.class) && @uri == other.uri
end

#hashObject



50
51
52
# File 'lib/bolt/target.rb', line 50

def hash
  @uri.hash ^ @options.hash
end

#hostObject



58
59
60
# File 'lib/bolt/target.rb', line 58

def host
  @uri_obj.hostname
end

#nameObject

name is currently just uri but should be used instead to identify the Target ouside the transport or uri options.



64
65
66
# File 'lib/bolt/target.rb', line 64

def name
  uri
end

#passwordObject



82
83
84
85
86
# File 'lib/bolt/target.rb', line 82

def password
  Addressable::URI.unencode_component(
    @uri_obj.password || @password
  )
end

#portObject



68
69
70
# File 'lib/bolt/target.rb', line 68

def port
  @uri_obj.port || @port
end

#protocolObject



72
73
74
# File 'lib/bolt/target.rb', line 72

def protocol
  @uri_obj.scheme || @protocol
end

#to_sObject



54
55
56
# File 'lib/bolt/target.rb', line 54

def to_s
  "Target('#{@uri}', #{@options})"
end

#update_conf(conf) ⇒ Object



20
21
22
23
24
25
26
27
28
29
30
31
32
33
# File 'lib/bolt/target.rb', line 20

def update_conf(conf)
  @protocol = conf[:transport]

  t_conf = conf[:transports][protocol.to_sym]
  # Override url methods
  @user = t_conf['user']
  @password = t_conf['password']
  @port = t_conf['port']

  url_keys = %w[user password port]
  @options = t_conf.reject { |k, _| url_keys.include?(k) }.merge(@options)

  self
end

#userObject



76
77
78
79
80
# File 'lib/bolt/target.rb', line 76

def user
  Addressable::URI.unencode_component(
    @uri_obj.user || @user
  )
end